(function ($) {
    $.fn.autogrow = function (options) {
        var defaults = {
            expandTolerance: 1,
            heightKeeperFunction: null,
            afterResize: function () {
            }
        };
        options = $.extend(defaults, options);
        var resizeConversationListHeight = function (obj) {
            var chatboxId = $(obj).parents('.chat-box').attr('chatboxId');
            var mainObj = $('.chat-box[chatboxId=' + chatboxId + ']');

            var miscDivHeight = mainObj.find('.miscDiv').height();
            var categoryPlaceHeight = mainObj.find('.category-place').height();
            var bodySliceHeight = mainObj.find('.chat-slice-body').height();
            var textPlace = mainObj.find('.textPlace').height();
            mainObj.find('.conversationList').height(bodySliceHeight - miscDivHeight - categoryPlaceHeight - textPlace);
            mainObj.find('.conversationList').css('bottom', categoryPlaceHeight);

            mainObj.find('.istyping').css('bottom', textPlace);
            mainObj.find('.conversationList').css('top', miscDivHeight);


            // scroll down
            var conversationList = mainObj.find('.conversationList .discussion');

            if (conversationList.hasClass('discussion')) {
                conversationList.animate({
                    scrollTop: conversationList[0].scrollHeight - conversationList.height()
                }, 1);
            }
        };

        // IE and Opera should never set a textarea height of 0px
        var hCheck = !($.browser.msie || $.browser.opera);
        function resize(e) {
            var $e = $(e.target || e), // event or element
                    contentLength = $e.val().length,
                    elementWidth = $e.innerWidth();
            if ($e.is(":hidden")) {
                // Do not do anything if the element is hidden as we cannot determine the height correctly
                return $e;
            }
            if (contentLength != $e.data("autogrow-length") || elementWidth != $e.data("autogrow-width")) {

                // For non-IE and Opera browser, it requires setting the height to 0px to compute the right height
                if (hCheck && (contentLength < $e.data("autogrow-length") ||
                        elementWidth != $e.data("autogrow-width"))) {
                    if ($.isFunction(options.heightKeeperFunction)) {
                        (options.heightKeeperFunction($e)).height((options.heightKeeperFunction($e)).height());
                    }
                    $e.css("height", "0px");
                }

                var height = Math.max($e.data("autogrow-min"), Math.ceil(Math.min(
                        $e.prop("scrollHeight") + options.expandTolerance * $e.data("autogrow-line-height"),
                        $e.data("autogrow-max"))));

                $e.css("overflow", ($e.prop("scrollHeight") > height ? "auto" : "hidden"));
                $e.css("height", height + "px");
                if ($.isFunction(options.heightKeeperFunction)) {
                    (options.heightKeeperFunction($e)).css({height: 'auto'});
                }
            }
            resizeConversationListHeight($e);
            options.afterResize($e);
            return $e;
        }
        ;

        function parseNumericValue(v) {
            var n = parseInt(v, 10);
            return isNaN(n) ? null : n;
        }
        ;

        function initElement($e) {
            $e.data("autogrow-min", options.minHeight || parseNumericValue($e.css('min-height')) || 0);
            $e.data("autogrow-max", options.maxHeight || parseNumericValue($e.css('max-height')) || 99999);
            $e.data("autogrow-line-height", options.lineHeight || parseNumericValue($e.css('line-height')));
            resize($e);
        }
        ;

        this.each(function () {
            var $this = $(this);

            if (!$this.data("autogrow-initialized")) {
//                            $this.css("padding-top", 0).css("padding-bottom", 0);
                $this.bind("keyup", resize).bind("focus", resize);
                $this.data("autogrow-initialized", true);
            }

            initElement($this);
            // Sometimes the CSS attributes are not yet there so the above computation might be wrong
            // 100ms delay will do the job
            setTimeout(function () {
                initElement($this);
            }, 100);
        });

        return this;
    };
})(jQuery);
var UGCM = {
    lz: function (i, c)
    {
        if (typeof c != 'number' || c <= 0 || (typeof i != 'number' && typeof i != 'string'))
        {
            return i;
        }
        i += '';
        while (i.length < c)
        {
            i = '0' + i;
        }
        return i;
    },
    getHashCode: function (s)
    {
        var hash = 0, c = (typeof s == 'string') ? s.length : 0, i = 0;
        while (i < c)
        {
            hash = ((hash << 5) - hash) + s.charCodeAt(i++);
        }

        return (hash < 0) ? ((hash * -1) + 0xFFFFFFFF) : hash;
    },
    uniqueId: function (s) {

        var i = this.getHashCode(s);
        return i.toString(32) + '-' + this.lz((s.length * 4).toString(16), 3);
    }
};

var resourceManager = {
    addCss: function (href, type) {
        /* 
         * check css id for redundency occuring cases 
         */
        var id = UGCM.uniqueId(href);
        if (!document.getElementById(id))
        {
            var head = document.getElementsByTagName('head')[0];
            var link = document.createElement('link');
            link.id = id;
            link.type = type;
            link.href = href;
            link.rel = 'stylesheet';
            link.media = 'all';
            head.appendChild(link);
        }
    },
    addJs: function (src, type, id) {
        /* 
         * check js id for redundency occuring cases 
         */
        if (typeof id !== 'string') {
            id = UGCM.uniqueId(src);
        }
        if (!document.getElementById(id))
        {
            var head = document.getElementsByTagName('head')[0];
            var js = document.createElement("script");
            js.id = id;
            js.type = type;
            js.src = src;
            js.async = true;
            head.appendChild(js);
            return true;
        } else {
            return false;
        }
    },
    addHtml: function (content) {
        $('body').append(content);
        return true;
    }
}
var mercury = {
    traceDump: false,
    trace: function (title, obj, error) {
        if (this.traceDump) {
            if (error) {
                if (obj) {
                    console.error(title, obj);
                } else {
                    console.error(title);
                }
            } else {
                if (obj) {
                    console.log(title, obj);
                } else {
                    console.log(title);
                }
            }
        }
    },
    initRequest: function (success, fail) {
        var jqxhr = $.getJSON("/mercury.php", {
            cmd: 'init'
        }, function (data) {
            success(data);
        }).fail(function () {
            fail();
        });
    },
    initChaparRequest: function (success, fail) {
        var jqxhr = $.getJSON("/mercury.php", {
            cmd: 'initChapar'
        }, function (data) {
            success(data);
        }).fail(function () {
            fail();
        });
    },
    init: function () {
        mercury.trace("mercury Init ...");
        this.initRequest(function (data) {
            if (parseInt(data.status) === 200) {
                for (var x in data.resources.styles) {
                    resourceManager.addCss(data.resources.styles[x].href, data.resources.styles[x].type)
                }
                for (var x in data.resources.scripts) {
                    var id = null;
                    if ('id' in data.resources.scripts[x]) {
                        id = data.resources.scripts[x].id;
                    }
                    resourceManager.addJs(data.resources.scripts[x].src, data.resources.scripts[x].type, id)
                }

                resourceManager.addHtml(data.body)

            }
        }, function () {
            mercury.trace("mercury Init: failed, retry in 5 sec later");
            setTimeout(function () {
                mercury.init();
            }, 5000);
        });
    },
    initChapar: function () {
        mercury.chatbox.buddyList.renderWaiting();
        mercury.trace("mercury chapar Init ...");
        this.initChaparRequest(function (data) {
            if (parseInt(data.status) === 200) {
                mercury.chatbox.buddyList.buddyList = data.buddyList;
                mercury.chatbox.buddyList.involvedList = data.involvedList;
                mercury.chatbox.buddyList.init();
            }
        }, function () {
            mercury.trace("mercury chapar Init : failed, retry in 5 sec later");
            setTimeout(function () {
                mercury.initChapar();
            }, 5000);
        });
    },
    chatbox: {
        requests: {
            sendRoot: function (section, part, action, params, success, fail) {
                var requestParams = {
                    cmd: 'mercury.root',
                    section: section,
                    part: part,
                    action: action,
                    params: params
                };
                if (typeof params === 'object') {
                    requestParams['params'] = params;
                }
                mercury.trace("mercury chapar request ...", requestParams);
                setTimeout(function () {

                    var jqxhr = $.ajax({
                        url: "/mercury.php",
                        dataType: 'json',
                        async: true,
                        data: requestParams,
                        success: function (data) {
                            mercury.trace("mercury chapar request: Done", requestParams);
                            if (typeof success === 'function') {
                                success(data);
                            }
                        }
                    }).fail(function () {
                        mercury.trace("mercury chapar request: Failed", requestParams);
                        if (typeof fail === 'function') {
                            fail();
                        }
                    });

                }, 50);
            },
            minimize: function (params) {
                this.sendRoot('chatbox', 'behavior', 'doMinimize', params);
            },
            maximize: function (params) {
                this.sendRoot('chatbox', 'behavior', 'doMaximize', params);
            },
            mute: function (params) {
                this.sendRoot('chatbox', 'behavior', 'doMute', params);
            },
            unMute: function (params) {
                this.sendRoot('chatbox', 'behavior', 'doUnMute', params);
            },
            closeBox: function (params) {
                this.sendRoot('chatbox', 'behavior', 'doCloseBox', params);
            },
            involve: function (params, success, fail) {
                this.sendRoot('chatbox', 'involvement', 'doInvolve', params, success, fail);
            },
            sendMessage: function (params, success, fail) {
                this.sendRoot('chatbox', 'involvement', 'doSendMessage', params, success, fail);
            },
            fetchMoreMessages: function (params, success, fail) {
                this.sendRoot('chatbox', 'involvement', 'fetchMoreMessages', params, success, fail);
            },
            isTypingRequest: function (params) {
                this.sendRoot('chatbox', 'involvement', 'isTyping', params);
            }
        },
        getParentBoxId: function (obj) {

            var chatboxId = $(obj).parents('.chat-box').attr('chatboxId');
            return '.chat-box[chatboxId=' + chatboxId + ']';
        },
        getBoxId: function (obj) {
            return $(obj).parents('.chat-box').attr('chatboxId');
        },
        getBoxName: function (obj) {
            return $(obj).parents('.chat-box').attr('chatboxName');
        },
        minimize: function (obj, visualOnly) {
            if (!visualOnly) {
                mercury.chatbox.requests.minimize({
                    'boxName': this.getBoxName(obj),
                    'boxId': this.getBoxId(obj)
                });
            }

            var parent = $(this.getParentBoxId(obj));

//            parent.animate({
//                bottom: -337
//            });
            parent.css('bottom', '63px');
            $(obj).find('.glyphicon').removeClass('glyphicon-minus');
            $(obj).find('.glyphicon').addClass('glyphicon-plus');
            $(obj).attr('onclick', 'javascript:mercury.chatbox.maximize(this)');
            parent.attr('chatboxStatus', 'min');

            mercury.chatbox.reQueue();
        },
        maximize: function (obj, visualOnly) {

            if (!visualOnly) {
                mercury.chatbox.requests.maximize({
                    'boxName': this.getBoxName(obj),
                    'boxId': this.getBoxId(obj)
                });
            }
            var parent = $(this.getParentBoxId(obj));
//            parent.animate({
//                bottom: 0
//            });

            parent.css('bottom', '400px');
            $(obj).find('.glyphicon').removeClass('glyphicon-plus');
            $(obj).find('.glyphicon').addClass('glyphicon-minus');
            $(obj).attr('onclick', 'javascript:mercury.chatbox.minimize(this)');
            parent.attr('chatboxStatus', 'max');
            this.setBubleCount(this.getBoxId(obj), 0, false);

            mercury.chatbox.reQueue();
        },
        setBubleCount: function (chatboxId, value, add) {
            var parent = $('.chat-box[chatboxId=' + chatboxId + ']');
            var bubleObj = $('.chat-box[chatboxId=' + chatboxId + ']').find('.bubleCount');
            var currentValue = parseInt(bubleObj.html());
            if (parent.attr('chatboxStatus') == 'min') {

                if (add) {
                    bubleObj.html(currentValue + parseInt(value));
                } else {
                    bubleObj.html(parseInt(value));
                }

                if (parseInt(bubleObj.html()) > 0) {
                    bubleObj.show();
                } else {
                    bubleObj.hide();
                }
            } else {
                if (parent.hasClass('queueHide')) {
                    if (add) {
                        bubleObj.html(currentValue + parseInt(value));
                    } else {
                        bubleObj.html(parseInt(value));
                    }

                    if (parseInt(bubleObj.html()) > 0) {
                        bubleObj.show();
                    } else {
                        bubleObj.hide();
                    }
                } else {

                    bubleObj.html('0');
                    bubleObj.hide();
                }
            }

            mercury.chatbox.reQueue();
        },
        close: function (obj, visualOnly) {

            if (!visualOnly) {
                mercury.chatbox.requests.closeBox({
                    'boxName': this.getBoxName(obj),
                    'boxId': this.getBoxId(obj)
                });
            }
            var parent = $(this.getParentBoxId(obj));
            parent.remove();

            mercury.chatbox.reQueue();
        },
        setting: function (obj, uid) {
            var parent = $(this.getParentBoxId(obj));
            if (typeof uid === 'number') {
                if ($(obj).parents('li[role=contactListLi]').index() > 2) {
                    parent.find('.dropdown').addClass('dropup');
                } else {
                    parent.find('.dropdown').removeClass('dropup');
                }
            } else {
                if (parseInt(parent.css('bottom')) === 400) {
                    parent.find('.dropdown').removeClass('dropup');
                } else {
                    parent.find('.dropdown').addClass('dropup');
                }
            }

        },
        mute: function (obj, uid, visualOnly) {
            $(obj).find('span').html('<span class="glyphicon glyphicon-volume-off" aria-hidden="false"></span>');
            if (typeof uid === 'number') {

                if (!visualOnly) {
                    mercury.chatbox.requests.mute({
                        'boxName': this.getBoxName(obj),
                        'boxId': this.getBoxId(obj),
                        'uid': uid
                    });
                }

                if (!(parseInt(uid) in mercury.chatbox.buddyList.muted.users)) {
                    mercury.chatbox.buddyList.muted.users[uid] = parseInt(uid);
                }
                $(obj).attr('onclick', 'javascript:mercury.chatbox.unMute(this, ' + uid + ');');
            } else {

                if (!visualOnly) {
                    mercury.chatbox.requests.mute({
                        'boxName': this.getBoxName(obj),
                        'boxId': this.getBoxId(obj)
                    });
                }
                mercury.chatbox.buddyList.muted.system = true;
                $(obj).attr('onclick', 'javascript:mercury.chatbox.unMute(this);');
            }
        },
        unMute: function (obj, uid, visualOnly) {
            $(obj).find('span').html('<span class="glyphicon glyphicon-volume-up" aria-hidden="false"></span>');
            $(obj).attr('onclick', 'javascript:mercury.chatbox.mute(this);');
            if (typeof uid === 'number') {

                if (!visualOnly) {
                    mercury.chatbox.requests.unMute({
                        'boxName': this.getBoxName(obj),
                        'boxId': this.getBoxId(obj),
                        'uid': uid
                    });
                }
                if (parseInt(uid) in mercury.chatbox.buddyList.muted.users) {
                    delete mercury.chatbox.buddyList.muted.users[uid];
                }
                $(obj).attr('onclick', 'javascript:mercury.chatbox.mute(this, ' + uid + ');');
            } else {

                if (!visualOnly) {
                    mercury.chatbox.requests.unMute({
                        'boxName': this.getBoxName(obj),
                        'boxId': this.getBoxId(obj)
                    });

                }

                mercury.chatbox.buddyList.muted.system = false;
                $(obj).attr('onclick', 'javascript:mercury.chatbox.mute(this);');
            }
        },
        buddyList: {
            muted: {
                system: false,
                users: {}
            },
            buddyList: {},
            involvedList: {},
            liProjectNameLimit: 29,
            currentType: 'e',
            init: function () {
                this.renderListUl();
            },
            changeCurrentType: function (type) {
                this.currentType = type;

                this.renderListUl();
            },
            renderListUl: function () {
                var tpl = _.template($('#mercury-contactlist-ul').html());
                $('.chat-box[chatboxname=buddyList]').find('.conversationList').html(tpl());
                this.renderListLi($('.chat-box[chatboxname=buddyList]').find('.conversationList').find('ul'));
            },
            renderListLi: function (obj) {
                var liHtml = '';
                for (var x in this.buddyList) {
                    var projectNamePure = this.buddyList[x].pn;
                    if (this.buddyList[x].pn.length > this.liProjectNameLimit) {
                        this.buddyList[x].pn = this.buddyList[x].pn.substring(0, this.liProjectNameLimit) + '...';
                    }
                    var tpl = _.template($('#mercury-contactlist-li').html());
                    var params = {
                        index: x,
                        type: this.buddyList[x].t,
                        projects_id: this.buddyList[x].pid,
                        projects_name: this.buddyList[x].pn,
                        projects_namePure: projectNamePure,
                        bids_id: this.buddyList[x].bid,
                        mute: this.buddyList[x].m
                    };

                    if (this.buddyList[x].t === 'e') {
                        params['username'] = this.buddyList[x].eun;
                        params['users_id'] = this.buddyList[x].euid;
                        params['picture'] = this.buddyList[x].eup;
                    } else {
                        params['username'] = this.buddyList[x].cun;
                        params['users_id'] = this.buddyList[x].cuid;
                        params['picture'] = this.buddyList[x].cup;
                    }
                    liHtml += tpl(params);
                    if (parseInt(this.buddyList[x].m) == 1) {
                        if (!(parseInt(params['users_id']) in mercury.chatbox.buddyList.muted.users)) {
                            mercury.chatbox.buddyList.muted.users[params['users_id']] = parseInt(params['users_id']);
                        }
                    }

                }

                $(obj).html(liHtml);

                this.doSearch();
                this.initInvolvers();
            },
            initInvolvers: function () {
                for (var x in this.involvedList) {
                    if (typeof this.involvedList[x] === 'object') {
                        var obj = $('.chat-box[chatboxname=buddyList]').find('li[buddyListProjects_id=' + this.involvedList[x].projects_id + '][buddyListBids_id=' + this.involvedList[x].bids_id + ']').find('.liInnerInvolveA');

                        mercury.chatbox.involve(obj, this.involvedList[x].status);
                        mercury.chatbox.reQueue();
                    }
                }
            },
            renderWaiting: function () {
                var tpl = _.template($('#mercury-loading').html());
                $('.chat-box[chatboxname=buddyList]').find('.conversationList').html(tpl());
            },
            doSearch: function (obj) {


                var employerCount = 0;
                var freelancerCount = 0;

                var employerSearchedCount = 0;
                var freelancerSearchedCount = 0;
                if (!obj) {
                    var searchText = $('#buddyListTextarea').val().trim();
                } else {
                    var searchText = $(obj).val().trim();
                }
                if (searchText.length > 0) {

//                    for (var x in this.buddyList) {
//                        var params = {
//                            index: x,
//                            type: this.buddyList[x].t,
//                            projects_id: this.buddyList[x].pid,
//                            bids_id: this.buddyList[x].bid
//                        };
//                        var filterObj = 'li[role=contactListLi][buddylisttype=' + params.type + '][buddylistprojects_id=' + params.projects_id + '][buddylistbids_id=' + params.bids_id + ']';
//
//                        if (this.buddyList[x].t !== this.currentType) {
//                            $(filterObj).addClass('hide');
//                        }else{
//                            $(filterObj).removeClass('hide');
//                        }
//                    }

                    $('li[role=contactListLi]').each(function (key, eachObj) {
                        $(eachObj).addClass('hide');
                    });

                    for (var x in this.buddyList) {
                        if (this.buddyList[x].t === 'e' && this.buddyList[x].t === this.currentType) {
                            if (this.buddyList[x].eun.substring(0, searchText.length) === searchText) {
                                $('li[buddylistindex=' + x + '][buddylisttype=' + this.buddyList[x].t + ']').removeClass('hide');
                            }
                        }

                        if (this.buddyList[x].t === 'c' && this.buddyList[x].t === this.currentType) {
                            if (this.buddyList[x].cun.substring(0, searchText.length) === searchText) {
                                $('li[buddylistindex=' + x + '][buddylisttype=' + this.buddyList[x].t + ']').removeClass('hide');
                            }
                        }

                        if (this.buddyList[x].t === 'e') {
                            if (this.buddyList[x].eun.substring(0, searchText.length) === searchText) {
                                employerSearchedCount++;
                            }
                        }

                        if (this.buddyList[x].t === 'c') {
                            if (this.buddyList[x].cun.substring(0, searchText.length) === searchText) {
                                freelancerSearchedCount++;
                            }
                        }
                    }

                    $('.employerListCount').html(employerSearchedCount);
                    $('.freelancerListCount').html(parseInt(freelancerSearchedCount));
                } else {
                    for (var x in this.buddyList) {
                        if (this.buddyList[x].t === 'e') {
                            employerCount++;
                        } else {
                            freelancerCount++;
                        }
                    }
                    $('li[role=contactListLi]').each(function (key, eachObj) {
                        $(eachObj).removeClass('hide');
                    });
                    $('.employerListCount').html(parseInt(employerCount));
                    $('.freelancerListCount').html(parseInt(freelancerCount));

                    for (var x in this.buddyList) {
                        var params = {
                            index: x,
                            type: this.buddyList[x].t,
                            projects_id: this.buddyList[x].pid,
                            bids_id: this.buddyList[x].bid
                        };
                        var filterObj = 'li[role=contactListLi][buddylisttype=' + params.type + '][buddylistprojects_id=' + params.projects_id + '][buddylistbids_id=' + params.bids_id + ']';

                        if (this.buddyList[x].t !== this.currentType) {
                            $(filterObj).addClass('hide');
                        } else {
                            $(filterObj).removeClass('hide');
                        }
                    }

                }


            }
        },
        getContactInfo: function (obj) {
            return $(obj).parents('li[role=contactListLi]');
        },
        bindTextBox: function (chatboxId) {
            $('div[chatboxId=' + chatboxId + ']').find('textarea').click();

        },
        requestInvolve: function (params, callback) {
            mercury.chatbox.requests.involve(params, function (data) {
                callback(data);
            }, function () {
                setTimeout(function () {
                    mercury.chatbox.requestInvolve(params, callback);
                }, 500);
            });
        },
        involve: function (obj, status) {


            if (parseInt(status) !== 1)
                status = 2;
            var contactInfo = this.getContactInfo(obj);
            var listProjectsId = contactInfo.attr('buddylistprojects_id');
            var listProjectName = contactInfo.attr('buddylistprojectName');
            var listBidsId = contactInfo.attr('buddylistbids_id');

            var listUsername = contactInfo.attr('buddyListUsername');
            var listUsersId = contactInfo.attr('buddyListUsers_id');


            this.involveRender(status, listProjectsId, listBidsId, listUsername, listUsersId, listProjectName, function (params) {
                mercury.chatbox.requestInvolve(params, function (data) {

                    var chatboxId = data.projects_id + '-' + data.bids_id + '-' + data.users_id;

                    $('.chat-inner-resp').find('div[chatboxId=' + chatboxId + ']').find('.discussLoading').remove();

                    mercury.chatbox.involveMessagesRender(chatboxId, data);
                    if (parseInt(data.messagesCount) > 20) {

                        $('div[chatboxId=' + chatboxId + ']').find('.fetchMore').show();
                    }
                    var discuss = $('div[chatboxId=' + chatboxId + ']').find('.discussion');
                    discuss.animate({
                        scrollTop: discuss[0].scrollHeight - discuss.height()
                    }, 1);
                    $('div[chatboxId=' + chatboxId + ']').find('textarea').focus();
                    mercury.chatbox.bindTextBox(chatboxId);
                });
            }, {
                'boxName': this.getBoxName(obj),
                'boxId': this.getBoxId(obj),
                'pid': listProjectsId,
                'bid': listBidsId,
                'status': status
            });

            mercury.chatbox.reQueue();

        },
        fetchMore: function (obj) {
            var parent = $(this.getParentBoxId(obj));
            var chatboxId = this.getBoxId(obj);
            mid = parent.find('li[mid]:first').attr('mid');
            var params = {
                'chatboxId': chatboxId,
                'mid': mid
            };
            mercury.chatbox.requests.fetchMoreMessages(params, function (data) {
                mercury.chatbox.involveMessagesRender(chatboxId, data, true);
            }, function () {

            });
        },
        isTypingStatusList: {},
        getUserTypingStatus: function (chatboxId) {

            return chatboxId in this.isTypingStatusList;
        },
        isTypingStatus: function (chatboxId, status) {
            var parent = $('div[chatboxId=' + chatboxId + ']');
            if (status) {
                parent.find('.istyping').removeClass('hide');

            } else {
                parent.find('.istyping').addClass('hide');
            }

            mercury.chatbox.reQueue();
        },
        isTypingRequest: function (chatboxId, status) {
            mercury.chatbox.requests.isTypingRequest({
                'chatboxId': chatboxId,
                'status': status
            });
        },
        involveRender: function (status, projects_id, bids_id, username, users_id, projectName, callback, callbackParams) {
            var tpl = _.template($('#mercury-chatbox-main').html());
            var params = {
                status: status,
                chatboxId: projects_id + '-' + bids_id + '-' + users_id,
                chatboxName: projects_id + '-' + bids_id + '-' + users_id,
                username: username,
                users_id: users_id,
                projects_id: projects_id,
                projectName: projectName
            };

            if ($('.chat-inner-resp').find('div[chatboxId=' + params.chatboxId + ']').length > 0) {

                var boxTmp = $('div[chatboxId=' + params.chatboxId + ']');

                $('div[chatboxId=' + params.chatboxId + ']').remove();

                $(boxTmp).insertAfter('.buddy-List');

                $('div[chatboxId=' + params.chatboxId + ']').find('textarea').focus();

                mercury.chatbox.bindTextBox(params.chatboxId);
            } else {
                $(tpl(params)).insertAfter('.buddy-List');
                this.bindTextBox(params.chatboxId);
                callback(callbackParams);
            }
            chapar.socket.callListener('onOnlineStatusRefresh');
        },
        newMessageRender: function (chatboxId, obj) {
            var tpl = _.template($('#mercury-chatbox-main-message-entry').html());

            var params = {
                self: ((parseInt(chapar.socket.uId) === parseInt(obj.uid)) ? true : false),
                photo: obj.photo,
                message: obj.message,
                users_id: obj.uid,
                username: obj.username,
                date: obj.date,
                mid: obj.mid
            };

            $('div[chatboxId=' + chatboxId + ']').find('.discussion').append(tpl(params));
            var discuss = $('div[chatboxId=' + chatboxId + ']').find('.discussion');
            discuss.animate({
                scrollTop: discuss[0].scrollHeight - discuss.height()
            }, 1);

            this.newMessageSoundNotify(obj.uid);
            if (parseInt(chapar.socket.uId) != parseInt(obj.uid)) {
                this.setBubleCount(chatboxId, 1, true);
            }
            this.isTypingStatus(chatboxId, false);

            mercury.chatbox.reQueue();
        },
        newMessageSoundNotify: function (uid) {
            if (parseInt(chapar.socket.uId) != parseInt(uid)) {
                if (!mercury.chatbox.buddyList.muted.system) {
                    if (!(parseInt(uid) in mercury.chatbox.buddyList.muted.users)) {
                        chapar.socket.callListener("onNewMessageSound");

                    }
                }
            }
        },
        involveMessagesRender: function (chatboxId, obj, appendBefore) {
            var tpl = _.template($('#mercury-chatbox-main-message-entry').html());
            var messages = '';
            var messageCount = obj.messages.length;
            for (var x in obj.messages) {
                var xIndex = (messageCount - 1) - x;
                var params = {
                    self: ((parseInt(chapar.socket.uId) === parseInt(obj.messages[xIndex].users_id)) ? true : false),
                    photo: obj.messages[xIndex].photo,
                    message: obj.messages[xIndex].message,
                    users_id: obj.messages[xIndex].users_id,
                    username: obj.messages[xIndex].username,
                    date: obj.messages[xIndex].date,
                    mid: obj.messages[xIndex].id,
                };
                messages += tpl(params);
            }
            if (appendBefore) {
                $(messages).insertAfter($('div[chatboxId=' + chatboxId + ']').find('.discussion').find('.fetchMore'));
                if (obj.messages.length < 20) {
                    $('div[chatboxId=' + chatboxId + ']').find('.discussion').find('.fetchMore').hide();
                }
            } else {
                $('div[chatboxId=' + chatboxId + ']').find('.discussion').append(messages);
                var discuss = $('div[chatboxId=' + chatboxId + ']').find('.discussion');
                discuss.animate({
                    scrollTop: discuss[0].scrollHeight - discuss.height()
                }, 1);
            }
        },
        involvedTextinput: function (obj, event, e) {
            var evt = e || window.event;
            var keyCode = evt.keyCode;
            var chatboxId = this.getBoxId(obj);
            var chatboxName = this.getBoxName(obj);


            if (event === 'keyDown') {
                if (keyCode === 13) {

                    var message = $(obj).val();

                    if (message.length > 0) {
                        mercury.chatbox.bindTextBox(chatboxId);
                        this.involvedSend(obj, message, chatboxId, chatboxName)
                    }

                    $(obj).val('');
                    $(obj).focus();

                }
            }

            if (event === 'keyUp') {
                if (keyCode === 13) {

                    $(obj).val('');
                    $(obj).focus();
                }

                if ($(obj).val().toString() != "") {
                    if (!this.getUserTypingStatus(chatboxId)) {
                        this.isTypingRequest(chatboxId, 1);
                    }
                    if (!(chatboxId in this.isTypingStatusList)) {
                        this.isTypingStatusList[chatboxId] = chatboxId;
                    }
                } else {
                    if (this.getUserTypingStatus(chatboxId)) {
                        this.isTypingRequest(chatboxId, 0);
                    }

                    if (chatboxId in this.isTypingStatusList) {
                        delete this.isTypingStatusList[chatboxId];
                    }
                }
            }
        },
        involvedSend: function (obj, message, chatboxId, chatboxName) {
            mercury.chatbox.requests.sendMessage({
                'boxId': chatboxId,
                'boxName': chatboxName,
                'message': message
            }, function () {
                // on success
            }, function () {
                // on fail
            });
        },
        reQueue: function () {
            var windowWidth = $(window).width();
            var chatboxWidth = 265;
            var queueboxWidth = $('.queue-box').width() + 15;

            var maxAvailableCount = parseInt((windowWidth - queueboxWidth) / chatboxWidth);
            var xCount = 0;
            $('.chat-box').each(function (key, obj) {
                if (++xCount <= maxAvailableCount) {
                    $(obj).show();
                    $(obj).removeClass('queueHide');

                } else {
                    $(obj).hide();
                    $(obj).addClass('queueHide');
                }
            });
            var queueCount = $('.queueHide').length;
            $('.queue-box').find('.conversationCount').html(queueCount);
            if (queueCount > 0) {
                $('.queue-box').removeClass('hide');
                var unreadCount = 0;
                $('.queueHide').each(function (key, obj) {
                    unreadCount += parseInt($(obj).find('.bubleCount').html());
                });


                if (parseInt(unreadCount) > 0) {
                    $('.queue-unreadCount').find('span').html(unreadCount);
                    $('.queue-unreadCount').show();
                } else {
                    $('.queue-unreadCount').hide();
                }

                var queueListItems = '';
                $('.queueHide').each(function (key, obj) {
                    var tpl = _.template($('#mercury-queueItem').html());

                    var uid = parseInt($(obj).find('.chatboxUid').html());
                    var username = $(obj).find('.chatboxUsername').html();
                    var projectName = $(obj).find('.chatboxProjectName').html();
                    var bubleCount = parseInt($(obj).find('.bubleCount').html());

                    var chatboxId = $(obj).attr('chatboxId');
                    var chatboxIdArray = chatboxId.toString().split('-');
                    var projects_id = parseInt(chatboxIdArray[0]);
                    var bids_id = parseInt(chatboxIdArray[1]);

                    var isTyping = !($(obj).find('.istyping').hasClass('hide'));

                    var params = {
                        projects_id: projects_id,
                        bids_id: bids_id,
                        uid: uid,
                        username: username,
                        projectName: projectName,
                        bubleCount: bubleCount,
                        isTyping: isTyping
                    };
                    queueListItems += tpl(params);
                });

                $('#mercury-queueList').find('.list-group').html(queueListItems);
            } else {
                $('.queue-box').addClass('hide');
            }


            chapar.socket.callListener('onOnlineStatusRefresh');
        }
    }
};
$(function () {
    mercury.init();
    $('textarea[max-rows-Allow=1]').autogrow({
        minHeight: 33,
        maxHeight: 114,
        expandTolerance: 0
    });
    $('textarea[max-rows-Allow=0]').autogrow({
        minHeight: 33,
        maxHeight: 33,
        expandTolerance: 0
    });
});